//Listing 10.11 Jini  Obtaining a reference for Lookup service and obtaining the reference at client side
//XML - A Distributed Computing Paradigm - Ajay M R

//java imports.
import java.util.*;
import java.io.*;

//RMI imports.
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

//JINI Package imports.
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;

/**
 * Class that act's as Client for invoking
 * getMessage method remotely on HelloWorldService
 * using JINI.
 *
 * @author : Ravinder Khokhar
 * @version : $1.0$
 */

public class JINIClient implements Runnable {

    // to facilitate serarch.
    protected ServiceTemplate template;

    //discover services.
    protected LookupDiscovery disco;

    /**
     * Inner class that will facilitate & take part in
     * the discovery protocol.
     */
    class Listener implements DiscoveryListener {

        /**
         * Invoked when new lookup service is available.
         * @param DisoveryEvent discovery event object which encapsulates
         *                      service registrars.
         */
        public void discovered( DiscoveryEvent event ) {

            ServiceRegistrar[] newregs = event.getRegistrars();

            for ( int i=0; i<newregs.length; i++) {

                lookForService( newregs[i] );

            }//for loop ends here.
        }//discovered method ends here.

        /**
         * called when we find a new lookup service.
         * @param DisoveryEvent discovery event object which encapsulates
         *                      service registrars.
         */
        public void discarded( DiscoveryEvent event) {
            //do nothing here.. since its the client.
        }//discard method ends here.

    }//Listner class ends here.


    /**
     * public constructor.
     */
    public JINIClient() throws IOException {

        //Initialization for query string. 
        Class[] types = {HelloWorldServiceInterface.class};
        
        //Initialize template to perform lookup search.
        template = new ServiceTemplate(null, types, null);

        //set the Security manager.
        if ( System.getSecurityManager() == null )
        System.setSecurityManager( new RMISecurityManager() );

        //search the group for available services.. in our case we'll search
        //for public groups only.
        disco = new LookupDiscovery( new String[] {""} );

        //Install a listner.
        disco.addDiscoveryListener( new Listener() );

    }//public constructor ends here.

    /**
     * Once we've found the lookup service, search for the proxy's
     * that implement HelloWorldServiceInterface.
     *
     * @param ServiceRegistrar service registrar object.
     * @return Object object received by marshalling after lookup.
     */
    protected Object lookForService(ServiceRegistrar registrar) {

        Object o = null;

        try {

            //the actual searching is done here by calling lookup.
            o = registrar.lookup( template );

        }catch (RemoteException ex){
            System.out.println("HelloWorldClient: Error in Lookup "+
            ex.getMessage() );
            return null;
        }

        if ( o == null ) {
            System.err.println("HelloWorldClient: No Matching Services");
            return null;
        }

        System.out.println("HelloWorldService: Got a matching Service "+
        "& invoking method on it");

        String output = ( (HelloWorldServiceInterface)o).getMessage();

        System.out.println("HelloWorldService: output after method "+
        "invocation - "+output);
        
        //exit the application since we do not need the object anymore.l
        //Note: this is true only in current scenario i.e. HelloWorld.
        System.exit(1);
        
        return o;

    }//lookForService method ends here.

    /**
     * this thread does nothing but keeps the VM from exiting
     * while we do discovery.
     */
    public void run(){

        while ( true ) {

            try {
                Thread.sleep(1000000);
        }catch(InterruptedException ex) {}

        }//while ends here.

    }//run ends here.


    /**
     * main entry point for this program.
     * <b> Steps :
     * - Instantiate our HelloWorldClient. <br>
     * - Start the background thread which will keep the VM running. <br>
     */
    public static void main(String[] args){

        try {
            
            JINIClient client = new JINIClient();
            new Thread( client).start();
            
        }catch( IOException ioex ){
            System.out.println("HelloWorldClient: Error - "+ioex.getMessage() );
        }//catch ends here.

    }//main method ends here.

}//HelloWorldClient class ends here.

