/* Static Model */
package sams;


import sams.LogInterface;
import sams.XMLDocumentInstance;
/* Connection Pool Manager is the main connection pool used to conenct to the system database. Connection pool calls may behave as Friends to Data Manager interfaces, and would be accessible by the same. */
public interface ConnectionPoolManagerInterface
{
	/* Initializes the connection pool. Usually Database Manager would call this method after setting various options like SetMaxConnections and SetDatabaseParameters. */
	public abstract boolean InitializeConnectionPool();
	/* Would stop the connections and refresh the pool by recreating the objects. This would typically be needed if the pool has being passed new values for the objects and those are to take into effect. */
	public abstract boolean RestartConnectionPool();
	/* Sets the limit to maximum number of connections to be created as needed.  */
	public abstract int SetMaxConnections();
	/* Stops the pool, the current connections are released. */
	public abstract boolean StopConnectionPool();
	/* Returns the current setting of Maximum allowable connections. */
	public abstract int GetCurrentMaxConnections();
	/* Returns the current total active connection objects in use. */
	public abstract int GetCurrentActiveConnections();
	/* Returns the boolean true or false depending on whether there is any connection available at present. */
	public abstract boolean IsConnectionFree();
	/* Sets the database parameters like DSN and authentication information. */
	public abstract boolean SetDatabaseParameters(char DatabaseDSNName,
											   char Username,
											   char Password);
	/* This method allows the connection pool to auto-set in case more than maximum allowed connections are needed. This is an example of coding some sort of knowledge base in the service module. The rule stream is simple XML, which specifies as under what circumstances would the rule is applicable. Using this attribute requires you to set the path for log file and the log obejct reference in this object. */
	public abstract boolean SetDemandForMoreRULE(XMLDocumentInstance RULEStream);
	/* Sets the log interface object reference to log the connection entries. */
	public abstract LogInterface SetLogInterfaceReference(LogInterface LogInterfaceReference);

}
/* END CLASS DEFINITION ConnectionPoolManagerInterface */
