/* Static Model */
package sams;


import sams.ErrorQueueInterface;
/* Represents the main start-up process for the entire system. */
public interface MainSystemInterface
{
	/* Initialize all the services required for the system to operate. This would typically refer to a configuration file, if certain services are not to be started with system. Usually this method would trigger all Initialize methods for other system objects. */
	public abstract boolean InitializeServices();
	/* Initializes the error queue object and return the reference to the same. This would allow for same error queue to be used for entire system, and would also provide the flexibility to construct a new object and use it. */
	public abstract ErrorQueueInterface GetErrorQueueReference();
	/* Shut down the system. This would invoke call to all shut down methods of obejcts, and would also trigger the persist level methods of queue objects. Again the settings could depend on configurations defined. */
	public abstract boolean ShutDownServices(double SecurityKey);
	/* Allows a soft-restart of the services. The reason for restarting the services through soft-trigger would be to reflect the change in system configuration settings. */
	public abstract boolean RestartServices(double SecurityKey);

}
/* END CLASS DEFINITION MainSystemInterface */
