/* Static Model */
package sams;


import sams.ErrorQueueInterface;
import sams.LogInterface;
import sams.TrackingInterface;
import sams.XMLDocumentInstance;
/* Queue Manager manager the queue in the system. It is the main interface to both IN and OUT message queues. As the system is composed of generic message packets deemed as instruction sets, the queues can be viewed as sequence of instructions that are interfaced and communicated to central processor through this object. */
public interface QueueManagerInterface
{
	/* Initialize Queue manager would trigger the initialize method of the actual Message Queue. The queues are formed and the configuration settings for number of messages to ne allowed and others would take effect here. */
	public abstract boolean InitializeQueueManager();
	/* This would persist the existing queue of messages, both IN and OUT, and dump them into Persist file. If no filename is provided it would dump in component root folder with default filename - PersistQueueINOUT.xml. */
	public abstract boolean PersistQueue();
	/* Persist queue dump file path and name references. */
	public abstract boolean PersistQueueFilePath(char FilePath,
											  char FileName);
	/* Would cause queue to re-start using soft-trigger. This would make the dynamically changed configuration settings to take effect, with exceptions of some parameters - Max queue size. As if this value is reduced then you have to shut down queue and re-initialize again. */
	public abstract boolean RefreshQueue();
	/* Refreshes or reconstructs the queue from persist file dump. */
	public abstract boolean RefreshQueueFromPersistData();
	/* Shuts the queue, this would cause the current queue actions to complete and would dump the entire IN/OUT queues in persist data storage file and stop all the queue services. */
	public abstract boolean ShutDownQueueManager();
	/* Soft-triggered method to restart the queue after it is shut down. */
	public abstract boolean RestartQueue();
	/* Get next queue message from IN Queue. */
	public abstract XMLDocumentInstance GetNextInMessage();
	/* Gets the requested OUT queue message. The message reference, Token#, is passed as parameter. */
	public abstract XMLDocumentInstance GetOutMessage(double MessageToken);
	/* Returns the queue message Token#, which can be used to retrieve the message from OUT queue. */
	public abstract double PutInMessage(XMLDocumentInstance MessagePacket);
	/* Allows a message to be put in OUT queue, this method wold be freind method to central processor, as no other system module or object would be allowed to access this method. Returns the message token on the message passed. Please note that the message placed in OUT queue has token defined within it. */
	public abstract double PutOutMessage(XMLDocumentInstance OutMessagePacket);
	/* Returns current messages in IN queue. */
	public abstract double GetCurrentINMessageCount();
	/* Returns current messages in OUT queue. */
	public abstract double GetCurrentOUTMessageCount();
	/* Sets the log interface obejct reference to log any events. */
	public abstract boolean SetLogInterfaceReference(LogInterface LogObjectReference);
	/* Set error queue reference. */
	public abstract boolean SetErrorQueueReference(ErrorQueueInterface ErrorQueueReference);
	/* Sets the archive file type, it can be either a database or file. */
	public abstract boolean SetArchiveType(int ArchiveType);
	/* Sets the archive database, please note that SetArchiveType needs to be called before this method and the ArchiveType has to be set to 1. */
	public abstract boolean SetArchiveDatabase(char DSNName,
											char Username,
											char Password);
	/* Sets the archive file, please note that SetArchiveType needs to be called before this method and the ArchiveType has to be set to 2. */
	public abstract boolean SetArchiveFilePath(char FilePath,
											char FileName);
	/* Sets reference to system wide Tracking object. */
	public abstract boolean SetTrackingReference(TrackingInterface TrackingObjectReference);

}
/* END CLASS DEFINITION QueueManagerInterface */
