/* Static Model */
package sams;


import sams.XMLDocumentInstance;
/* Provides security services to all the system objects. */
public interface SecurityInterface
{
	/* Generic Initialize method. Please note that SetSecurityDatabase could be called within this method if parameters are provided in the configuration file. */
	public abstract boolean InitializeSecurityServices();
	/* Sets the poll cycle, controls the thread trigger, which would refresh the system security policy at regular intervals. If the system administrator prefers no auto-refresh then set the value of time to -1. */
	public abstract boolean SecurityRefreshCycle(long RefreshTime);
	/* Sets the security database pointer. Passes DSN and authentication information for using a database. */
	public abstract boolean SetSecurityDatabase(char DatabaseDSN,
											 char Username,
											 char Password);
	/* Authenticates the request. A request is the whole message packet passed as-is, and this method uses the header user information to authenticate it from the database. */
	public abstract boolean ValidateRequest(XMLDocumentInstance RequestPacket);
	/* Allows a new user created in the system. The user information is passed as an XML document. */
	public abstract boolean CreateSecurityEntry(XMLDocumentInstance EntryXML);
	/* To update the user information in security database. Again the user information is passed as XML string, with suitable authentication. */
	public abstract boolean EditSecurityEntry(XMLDocumentInstance EditXMLDocument);
	/* To delete the user information in security database. Again the user information is passed as XML string, with suitable authentication. */
	public abstract boolean DeleteSecurityEntry(XMLDocumentInstance DeleteXML,
											 long SecurityKey);
	/* Sets the security level for authenticating the user requests. Usually there would be 3 levels of security. Level 1 would mean the authentication would verify security key and encrypted code. Level 2 would verify just private key and security key. Level 3 would verify only username [public key] and password [private key]. */
	public abstract boolean SetSecurityLevel(int LevelNumber);

}
/* END CLASS DEFINITION SecurityInterface */
